import argparse
import os.path
import subprocess
import tempfile
import sys


def create_key_pair(name, comment, email, expires):
    """
    Create a new key pair
    """
    gpg_dir = os.path.expanduser('~/.gnupg')
    with tempfile.NamedTemporaryFile(dir=gpg_dir) as params_file:
        print >> params_file, "Key-Type: RSA"
        print >> params_file, "Key-Length: 2048"
        print >> params_file, "Name-Real:", name
        if comment:
            print >> params_file, "Name-Comment:", comment
        if email:
            print >> params_file, "Name-Email:", email
        if expires:
            print >> params_file, "Expire-Date:", expires
        print >> params_file, "%commit"
        params_file.flush()

        keyid = None
        out = subprocess.check_output(['gpg', '--batch', '--gen-key',
                                       params_file.name],
                                      stderr=subprocess.STDOUT)
        for line in out.split('\n'):
            if 'marked as ultimately trusted' in line:
                _, _, keyid, _ = line.split(None, 3)
                break

        return keyid


def export_public_key(keyid, filename, armor=True):
    """
    Export the public key to a file
    """
    cmd = ['gpg', '--output', filename, '--export']
    cmd.append('--armor' if armor else '--no-armor')
    cmd.append(keyid)
    subprocess.check_call(cmd)


def archive_key_pair(args):
    print "FIXME: not implemented yet"


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="Create and archive signing keys")
    group = parser.add_mutually_exclusive_group()
    group.add_argument("--create",
                       metavar="DESC", help="Generate a new key pair")
    group.add_argument("--archive",
                       metavar="KEY", help="Retire a key pair")
    parser.add_argument("--export", help="Export public key to file")
    parser.add_argument("--email", default="secure@citrix.com",
                        help="E-mail address")
    parser.add_argument("--comment", default="XenServer Official Signing Key",
                        help="E-mail address")
    parser.add_argument("--expires", default=None,
                        help="Expires after period")
    parser.add_argument("--no-email", action="store_const", const=None,
                        dest="email")
    parser.add_argument("--no-comment", action="store_const", const=None,
                        dest="comment")

    return parser.parse_args(argv)


def main(argv):
    """
    Manage key pairs used for signing
    """
    args = parse_args_or_exit(argv)

    if args.create:
        keyid = create_key_pair(args.create, args.comment, args.email,
                                args.expires)
        if args.export:
            export_public_key(keyid, args.export)
    if args.archive:
        archive_key_pair(args)


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
