#!/usr/bin/env python

import argparse
import os
import os.path
import shutil
import sys

from update_package.common import *


def parse_args_or_exit(argv=None):
    """
    Parse command line options
    """
    parser = argparse.ArgumentParser(description="Unpack update package")
    parser.add_argument("update", metavar="ISO",
                        help="Update ISO to unpack")
    parser.add_argument("-o", "--output", metavar="DIR", required=True,
                        help="Output directory")

    return parser.parse_args(argv)


def main(argv):
    """
    Unpack the contents of an update to a directory
    """
    args = parse_args_or_exit(argv)
    update = fetch_remote_file(args.update)
    try:
        if not os.path.isdir(args.output):
            os.mkdir(args.output)
        unpack_archive(update, args.output)
    finally:
        if update != args.update:
            shutil.rmtree(os.path.dirname(update), ignore_errors=True)


def _main():
    """
    Entry point for setuptools CLI wrapper
    """
    main(sys.argv[1:])


# Entry point when run directly
if __name__ == "__main__":
    _main()
